/*
 * Copyright (C) 2017 C-SKY Microsystems Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/******************************************************************************
 * @file     example_cache.c
 * @brief    the main function for the cache example
 * @version  V1.0
 * @date     25. March 2017
 ******************************************************************************/

#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include <soc.h>
#include <csi_core.h>

#define TEST_BUF_SIZE 4096
static uint8_t __attribute__((aligned(TEST_BUF_SIZE))) test_buf[TEST_BUF_SIZE];

int example_cache(void)
{
    uint32_t access, miss;
    uint32_t i;

    csi_cache_set_range(3, (uint32_t)&test_buf, CACHE_CRCR_4K, 1);
    csi_dcache_enable();

    csi_cache_enable_profile();
    csi_cache_reset_profile();

    access = csi_cache_get_access_time();
    miss = csi_cache_get_miss_time();

    printf("before memory access:\n");
    printf("  cache access times: %u\n", access * 256);
    printf("  cache miss times: %u\n", miss * 256);

    for (i = 0; i < TEST_BUF_SIZE; i++) {
        volatile uint8_t __attribute__((unused)) unused = test_buf[i];
    }

    access = csi_cache_get_access_time();
    miss = csi_cache_get_miss_time();

    printf("after memory access:\n");
    printf("  cache access times: %u\n", access * 256);
    printf("  cache miss times: %u\n", miss * 256);

    printf("run cache example successfully.\n");
    return 0;
}

int main(void)
{
    return example_cache();
}
