/*
 * Copyright (C) 2017 C-SKY Microsystems Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/******************************************************************************
 * @file     sem_param.c
 * @brief    the main function
 * @version  V1.0
 * @date     15. July 2017
 ******************************************************************************/

#include <test_util.h>
#include "sem_test.h"

#define MODULE_NAME "sem_param"

static uint8_t sem_param_case1(void)
{
    k_status_t ret;

    test_sem = csi_kernel_sem_new(0, 0);
    MYASSERT(test_sem == NULL);

    test_sem = csi_kernel_sem_new(4, 5);
    MYASSERT(test_sem == NULL);

//    test_sem = csi_kernel_sem_new(0x7fffffff, 0x7fffffff);
//    MYASSERT(test_sem == NULL);

    ret = csi_kernel_sem_del(NULL);
    MYASSERT(ret == -EINVAL);

    test_sem = csi_kernel_sem_new(3, 0);
    MYASSERT(test_sem != NULL);

    ret = csi_kernel_sem_del(test_sem);
    MYASSERT(ret == K_OK);

    return 0;
}

static uint8_t sem_param_case2(void)
{
    k_status_t ret;

    test_sem = csi_kernel_sem_new(10, 1);
    MYASSERT(test_sem != NULL);

//    csi_kernel_sched_suspend();
//    ret = csi_kernel_sem_wait(test_sem, WAIT_FOREVER);
//    csi_kernel_sched_resume(0);
//    MYASSERT(ret == -EPERM);

    ret = csi_kernel_sem_del(test_sem);
    MYASSERT(ret == K_OK);

    return 0;
}

static uint8_t sem_param_case3(void)
{
    k_status_t ret;

    test_sem = csi_kernel_sem_new(10, 0);
    MYASSERT(test_sem != NULL);

    ret = csi_kernel_sem_del(test_sem);
    MYASSERT(ret == K_OK);

    return 0;
}

static uint8_t sem_param_case4(void)
{
    k_status_t   ret;
    int32_t      count;

    test_sem = csi_kernel_sem_new(10, 0);
    MYASSERT(test_sem != NULL);

    count = csi_kernel_sem_get_count(NULL);
    MYASSERT(count == -EINVAL);

    ret = csi_kernel_sem_wait(NULL, NO_WAIT);
    MYASSERT(ret == -EINVAL);

    ret = csi_kernel_sem_post(NULL);
    MYASSERT(ret == -EINVAL);

    ret = csi_kernel_sem_del(test_sem);
    MYASSERT(ret == K_OK);

    return 0;
}

static void task_sem_entry_private(void)
{
    sem_param_case1();
    sem_param_case2();
    sem_param_case3();
    sem_param_case4();

    test_case_success++;
    PRINT_RESULT(MODULE_NAME, PASS);

    next_test_case_notify();
    csi_kernel_task_del(csi_kernel_task_get_cur());
}

void sem_param_test(void)
{

    csi_kernel_task_new((k_task_entry_t)task_sem_entry_private, MODULE_NAME, 0, TASK_SEM_PRI, 0, NULL, TEST_CASE_TASK_SIZE, &task_sem);

    if (task_sem == NULL) {
        test_case_fail++;
        PRINT_RESULT(MODULE_NAME, FAIL);
    }

    next_test_case_wait();
}

